@@ -1,18 +1,26 @@
 import sys
+from typing import TYPE_CHECKING
 
 import ddtrace
+from ddtrace import config
 from ddtrace.constants import ANALYTICS_SAMPLE_RATE_KEY
 from ddtrace.ext import SpanTypes
 from ddtrace.ext import http
-from ddtrace.propagation.http import HTTPPropagator
-from ddtrace.settings import config
 
 from .. import trace_utils
-from ...compat import reraise
+from ...internal.compat import reraise
 from ...internal.logger import get_logger
 from .utils import guarantee_single_callable
 
 
+if TYPE_CHECKING:
+    from typing import Any
+    from typing import Mapping
+    from typing import Optional
+
+    from ddtrace import Span
+
+
 log = get_logger(__name__)
 
 config._add(
@@ -59,6 +67,12 @@ def _default_handle_exception_span(exc, span):
     span.set_tag(http.STATUS_CODE, 500)
 
 
+def span_from_scope(scope):
+    # type: (Mapping[str, Any]) -> Optional[Span]
+    """Retrieve the top-level ASGI span from the scope."""
+    return scope.get("datadog", {}).get("request_span")
+
+
 class TraceMiddleware:
     """
     ASGI application middleware that traces the requests.
@@ -86,13 +100,15 @@ class TraceMiddleware:
         if scope["type"] != "http":
             return await self.app(scope, receive, send)
 
-        headers = _extract_headers(scope)
-
-        if self.integration_config.distributed_tracing:
-            propagator = HTTPPropagator()
-            context = propagator.extract(headers)
-            if context.trace_id:
-                self.tracer.context_provider.activate(context)
+        try:
+            headers = _extract_headers(scope)
+        except Exception:
+            log.warning("failed to decode headers for distributed tracing", exc_info=True)
+            headers = {}
+        else:
+            trace_utils.activate_distributed_headers(
+                self.tracer, int_config=self.integration_config, request_headers=headers
+            )
 
         resource = "{} {}".format(scope["method"], scope["path"])
 
@@ -103,6 +119,8 @@ class TraceMiddleware:
             span_type=SpanTypes.WEB,
         )
 
+        scope["datadog"] = {"request_span": span}
+
         if self.span_modifier:
             self.span_modifier(span, scope)
 
@@ -159,4 +177,8 @@ class TraceMiddleware:
             self.handle_exception_span(exc, span)
             reraise(exc_type, exc_val, exc_tb)
         finally:
+            try:
+                del scope["datadog"]["request_span"]
+            except KeyError:
+                pass
             span.finish()
